import { RedBlackTreeIterator, RedBlackTreeStructure } from '../internals';
import { FindOp } from './find';
/**
 * Returns an iterator pointing at whichever entry in the tree which is closest to the specified input key. The logic
 * determining which entry to locate is controlled by the `op` parameter.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {FindOp} op The operation that determines which entry to find in the tree
 * @param {boolean} reverse If true, the iterator will iterate backward toward the first entry in the tree
 * @param {K} key A reference key used as input to the find operation
 * @param {RedBlackTreeStructure<K, V>} tree The input tree
 * @returns {RedBlackTreeIterator<K, V>} An iterator that retrieves each successive entry in the tree, starting from the
 *   matched entry. If no matching entry is found, an empty iterator is returned.
 */
export declare function iterateFrom<K, V = null>(op: FindOp, reverse: boolean, key: K, tree: RedBlackTreeStructure<K, V>): RedBlackTreeIterator<K, V>;
