"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@collectable/core");
const internals_1 = require("../internals");
/**
 * Retrieves the key at the specified index (ordinal) in the tree. If a negative number is specified for the index,
 * the number is treated as a backtracking offset, with -1 matching the last element in the list, -2 matching the
 * second-last, and so forth.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {number} index The numeric index of the entry to retrieve (A negative number counts as a backtracking offset from the end of the tree)
 * @param {RedBlackTreeStructure<K, V>} tree The input tree
 * @returns {(K|undefined)} The key at the specified index, or undefined if the index was out of range
 */
function keyAt(index, tree) {
    const node = (index = core_1.normalizeIndex(index, tree._size)) === -1 ? void 0 : internals_1.findByIndex(index, tree);
    return core_1.isDefined(node) ? node.key : void 0;
}
exports.keyAt = keyAt;
//# sourceMappingURL=keyAt.js.map