"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const C = require("@collectable/core");
const internals_1 = require("../internals");
const internals_2 = require("../internals");
/**
 * Adds a new key and value to the tree, or updates the value if the key was previously absent from the tree. If the new
 * value is the equal to a value already associated with the specified key, no change is made, and the original tree is
 * returned.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {K} key The key of the entry to be updated or inserted
 * @param {V} value The value that should be associated with the key
 * @param {RedBlackTreeStructure<K, V>} tree The tree to be updated
 * @returns {RedBlackTreeStructure<K, V>} An updated copy of the tree, or the same tree if the input tree was already mutable
 */
function set(key, value, tree) {
    var originalTree = tree;
    tree = C.modify(tree);
    if (tree._size === 0) {
        tree._root = internals_2.createNode(tree, false, key, value);
        tree._size = 1;
        return C.commit(tree);
    }
    tree._root = C.modifyAsSubordinate(tree, tree._root);
    var p = internals_2.findPath(tree, key, tree._root, tree._compare);
    if (p.next === 0 /* NONE */) {
        var replaced = internals_2.assignValue(value, p.node);
        internals_1.PathNode.release(p, tree._root);
        if (!replaced) {
            tree = originalTree;
        }
    }
    else {
        var node = internals_2.createNode(tree, true, key, value);
        internals_2.setChild(p.next, p.node, node);
        if (tree._size > 1) {
            internals_2.rebalance(p, node, p.node, tree);
        }
        tree._size++;
    }
    return C.commit(tree);
}
exports.set = set;
//# sourceMappingURL=set.js.map