"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@collectable/core");
const node_1 = require("./node");
const unwrap_1 = require("./unwrap");
const functions_1 = require("../functions");
class RedBlackTreeStructure {
    /** @internal */
    constructor(mctx, compare, root, size) {
        this['@@mctx'] = mctx;
        this._compare = compare;
        this._root = root;
        this._size = size;
    }
    /** @internal */
    get '@@is-collection'() { return true; }
    /** @internal */
    get '@@size'() { return this._size; }
    /** @internal */
    '@@clone'(mctx) {
        return new RedBlackTreeStructure(mctx, this._compare, this._root, this._size);
    }
    /** @internal */
    '@@equals'(other) {
        return functions_1.isEqual(this, other);
    }
    /** @internal */
    '@@hash'() {
        return core_1.hashIterator(functions_1.iterateFromFirst(this));
    }
    /** @internal */
    '@@unwrap'() {
        return core_1.unwrap(this);
    }
    /** @internal */
    '@@unwrapInto'(target) {
        return unwrap_1.unwrap(this, target);
    }
    /** @internal */
    '@@createUnwrapTarget'() {
        return {};
    }
    /** @internal */
    '@@get'(key) {
        return functions_1.get(key, this);
    }
    /** @internal */
    '@@has'(key) {
        return functions_1.has(key, this);
    }
    /** @internal */
    '@@set'(key, value) {
        return functions_1.set(key, value, this);
    }
    /** @internal */
    '@@update'(updater, key) {
        return functions_1.update(updater, key, this);
    }
    /** @internal */
    '@@verifyKey'(key) {
        return core_1.isDefined(key);
    }
    [Symbol.iterator]() {
        return functions_1.iterateFromFirst(this);
    }
}
exports.RedBlackTreeStructure = RedBlackTreeStructure;
function isRedBlackTree(arg) {
    return core_1.isObject(arg) && arg instanceof RedBlackTreeStructure;
}
exports.isRedBlackTree = isRedBlackTree;
function createTree(compare, pctx) {
    return new RedBlackTreeStructure(core_1.selectContext(pctx), compare, node_1.NONE, 0);
}
exports.createTree = createTree;
function cloneTree(tree, pctx) {
    if (core_1.isUndefined(pctx))
        pctx = core_1.isMutable(tree);
    return new RedBlackTreeStructure(core_1.selectContext(pctx), tree._compare, tree._root, tree._size);
}
exports.cloneTree = cloneTree;
//# sourceMappingURL=RedBlackTree.js.map