"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@collectable/core");
const node_1 = require("./node");
const path_1 = require("./path");
function findNodeByKey(key, tree) {
    var node = tree._root, compare = tree._compare, found = false;
    var loopCounter = 0; // ## DEV ##
    while (!node_1.isNone(node) && !found) {
        // ## DEV [[
        if (++loopCounter === 100) {
            throw new Error('Loop never terminated');
        }
        // ]] ##
        var c = compare(key, node.key);
        if (c === 0) {
            found = true;
        }
        else {
            node = c > 0 ? node._right : node._left;
        }
    }
    return found ? node : void 0;
}
exports.findNodeByKey = findNodeByKey;
function findIndex(key, node, compare) {
    var found = false, i = node._left._count;
    var loopCounter = 0; // ## DEV ##
    while (!node_1.isNone(node) && !found) {
        // ## DEV [[
        if (++loopCounter === 100) {
            throw new Error('Loop never terminated');
        }
        // ]] ##
        var c = compare(key, node.key);
        if (c === 0) {
            found = true;
        }
        else if (c > 0) {
            node = node._right;
            i += node._left._count + 1;
        }
        else {
            node = node._left;
            i -= (node._right._count + 1);
        }
    }
    return found ? i : -1;
}
exports.findIndex = findIndex;
function findPathToNodeByKey(key, node, compare) {
    var path = path_1.PathNode.NONE, found = false;
    var loopCounter = 0; // ## DEV ##
    while (!node_1.isNone(node) && !found) {
        // ## DEV [[
        if (++loopCounter === 100) {
            throw new Error('Loop never terminated');
        }
        // ]] ##
        var c = compare(key, node.key);
        path = path_1.PathNode.next(node, path, 0 /* NONE */);
        if (c === 0) {
            found = true;
        }
        else if (c > 0) {
            node = node._right;
            path.next = 2 /* RIGHT */;
        }
        else {
            node = c > 0 ? node._right : node._left;
            path.next = 1 /* LEFT */;
        }
    }
    if (!found) {
        path_1.PathNode.release(path);
        path = path_1.PathNode.NONE;
    }
    return path;
}
exports.findPathToNodeByKey = findPathToNodeByKey;
function findMaxNodeLeftOfKey(allowExactKeyMatch, key, tree) {
    var node = tree._root, compare = tree._compare, found = node_1.NONE, done = false;
    var loopCounter = 0; // ## DEV ##
    while (!node_1.isNone(node) && !done) {
        // ## DEV [[
        if (++loopCounter === 100) {
            throw new Error('Loop never terminated');
        }
        // ]] ##
        var c = compare(key, node.key);
        if (c === 0) {
            done = allowExactKeyMatch;
            if (done) {
                found = node;
            }
            else {
                node = node._left;
            }
        }
        else if (c > 0) {
            found = node;
            node = node._right;
        }
        else {
            node = node._left;
        }
    }
    return node_1.isNone(found) ? void 0 : found;
}
exports.findMaxNodeLeftOfKey = findMaxNodeLeftOfKey;
function findPathToMaxNodeLeftOfKey(allowExactKeyMatch, key, node, parent, compare) {
    var path, found, c = compare(key, node.key);
    if (c <= 0) {
        if (c === 0 && allowExactKeyMatch) {
            return path_1.PathNode.next(node, parent, 0 /* NONE */);
        }
        if (node_1.isNone(node._left)) {
            return void 0;
        }
        path = path_1.PathNode.next(node, parent, 1 /* LEFT */);
        found = findPathToMaxNodeLeftOfKey(allowExactKeyMatch, key, node._left, path, compare);
        if (core_1.isUndefined(found)) {
            path.release();
            return void 0;
        }
        return found;
    }
    if (node_1.isNone(node._right)) {
        return path_1.PathNode.next(node, parent, 0 /* NONE */);
    }
    path = path_1.PathNode.next(node, parent, 2 /* RIGHT */);
    found = findPathToMaxNodeLeftOfKey(allowExactKeyMatch, key, node._right, path, compare);
    if (core_1.isUndefined(found)) {
        path.next = 0 /* NONE */;
        return path;
    }
    return found;
}
exports.findPathToMaxNodeLeftOfKey = findPathToMaxNodeLeftOfKey;
function findMinNodeRightOfKey(allowExactKeyMatch, key, node, compare) {
    var c = compare(key, node.key);
    if (c >= 0) {
        if (c === 0 && allowExactKeyMatch) {
            return node;
        }
        if (node_1.isNone(node._right)) {
            return void 0;
        }
        return findMinNodeRightOfKey(allowExactKeyMatch, key, node._right, compare);
    }
    if (node_1.isNone(node._left)) {
        return node;
    }
    var found = findMinNodeRightOfKey(allowExactKeyMatch, key, node._left, compare);
    return core_1.isUndefined(found) ? node : found;
}
exports.findMinNodeRightOfKey = findMinNodeRightOfKey;
function findPathToMinNodeRightOfKey(allowExactKeyMatch, key, node, parent, compare) {
    var path, found, c = compare(key, node.key);
    if (c >= 0) {
        if (c === 0 && allowExactKeyMatch) {
            return path_1.PathNode.next(node, parent, 0 /* NONE */);
        }
        if (node_1.isNone(node._right)) {
            return void 0;
        }
        path = path_1.PathNode.next(node, parent, 2 /* RIGHT */);
        found = findPathToMinNodeRightOfKey(allowExactKeyMatch, key, node._right, path, compare);
        if (core_1.isUndefined(found)) {
            path.release();
            return void 0;
        }
        return found;
    }
    if (node_1.isNone(node._left)) {
        return path_1.PathNode.next(node, parent, 0 /* NONE */);
    }
    path = path_1.PathNode.next(node, parent, 1 /* LEFT */);
    found = findPathToMinNodeRightOfKey(allowExactKeyMatch, key, node._left, path, compare);
    if (core_1.isUndefined(found)) {
        path.next = 0 /* NONE */;
        return path;
    }
    return found;
}
exports.findPathToMinNodeRightOfKey = findPathToMinNodeRightOfKey;
function findByIndex(index, tree) {
    var node = tree._root;
    var i = node._left._count;
    var loopCounter = 0; // ## DEV ##
    while (i !== index) {
        // ## DEV [[
        if (++loopCounter === 100) {
            throw new Error('Loop never terminated');
        }
        // ]] ##
        if (index < i) {
            node = node._left;
            i -= (node._right._count + 1);
        }
        else {
            node = node._right;
            i += node._left._count + 1;
        }
    }
    return node;
}
exports.findByIndex = findByIndex;
function findPathToIndex(index, node) {
    var path = path_1.PathNode.NONE;
    var i = node._left._count;
    var loopCounter = 0; // ## DEV ##
    while (i !== index) {
        // ## DEV [[
        if (++loopCounter === 100) {
            throw new Error('Loop never terminated');
        }
        // ]] ##
        if (index < i) {
            path = path_1.PathNode.next(node, path, 1 /* LEFT */);
            node = node._left;
            i -= (node._right._count + 1);
        }
        else {
            path = path_1.PathNode.next(node, path, 2 /* RIGHT */);
            node = node._right;
            i += node._left._count + 1;
        }
    }
    return path_1.PathNode.next(node, path, 0 /* NONE */);
}
exports.findPathToIndex = findPathToIndex;
function leftOf(node) {
    return node._left;
}
function rightOf(node) {
    return node._right;
}
function findNext(compare, reversed, inclusive, key, current) {
    let alreadyVisited, findPath, childOf;
    if (reversed) {
        alreadyVisited = 1 /* LEFT */;
        findPath = findPathToMaxNodeLeftOfKey;
        childOf = leftOf;
    }
    else {
        alreadyVisited = 2 /* RIGHT */;
        findPath = findPathToMinNodeRightOfKey;
        childOf = rightOf;
    }
    if (inclusive && compare(key, current.node.key) === 0) {
        inclusive = false;
    }
    let found, tentativeParentMatch = false;
    while (current.isActive() && core_1.isUndefined(found)) {
        const parent = current.parent;
        if (!parent.isNone()) { // check if the parent is in the direction we want to go
            let c = compare(key, parent.node.key);
            if (reversed)
                c = -c;
            if (c >= 0) { // the parent matches or precedes the search key
                current = current.release();
                current.next = alreadyVisited; // keep a record of the already-traversed branch for iteration purposes
                if (inclusive && c === 0) {
                    found = current;
                }
                continue;
            }
            else {
                tentativeParentMatch = true;
            }
        }
        const child = childOf(current.node);
        if (!node_1.isNone(child)) {
            found = findPath(inclusive, key, child, current, compare);
        }
        if (core_1.isUndefined(found)) {
            current = current.release();
            if (tentativeParentMatch) {
                found = current;
            }
        }
        current.next = alreadyVisited; // keep a record of the already-traversed branch for iteration purposes
    }
    return found;
}
exports.findNext = findNext;
//# sourceMappingURL=find.js.map