import { findMaxNodeLeftOfKey, findMinNodeRightOfKey, findNodeByKey } from '../internals';
/**
 * Returns the entry in the tree which is closest to the specified input key. The logic determining which entry to
 * locate is controlled by the `op` parameter.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {FindOp} op The operation that determines which entry to find in the tree
 * @param {K} key A reference key used as input to the find operation
 * @param {RedBlackTreeStructure<K, V>} tree The input tree
 * @returns {(RedBlackTreeEntry<K, V>|undefined)} The entry matching the specified key and operation, or undefined if not found
 */
export function find(op, key, tree) {
    var exact = false;
    if (tree._size > 0) {
        switch (op) {
            case 'gte': exact = true;
            case 'gt':
                return findMinNodeRightOfKey(exact, key, tree._root, tree._compare);
            case 'lte': exact = true;
            case 'lt':
                return findMaxNodeLeftOfKey(exact, key, tree);
            case 'eq':
                return findNodeByKey(key, tree);
        }
        throw new Error(`Invalid find operation; must be 'lt', 'lte', 'gt', 'gte' or 'eq'`);
    }
}
//# sourceMappingURL=find.js.map