import { hashIterator, isDefined, isMutable, isObject, isUndefined, selectContext, unwrap as _unwrap } from '@collectable/core';
import { NONE } from './node';
import { unwrap } from './unwrap';
import { get, has, isEqual, iterateFromFirst, set, update } from '../functions';
export class RedBlackTreeStructure {
    /** @internal */
    constructor(mctx, compare, root, size) {
        this['@@mctx'] = mctx;
        this._compare = compare;
        this._root = root;
        this._size = size;
    }
    /** @internal */
    get '@@is-collection'() { return true; }
    /** @internal */
    get '@@size'() { return this._size; }
    /** @internal */
    '@@clone'(mctx) {
        return new RedBlackTreeStructure(mctx, this._compare, this._root, this._size);
    }
    /** @internal */
    '@@equals'(other) {
        return isEqual(this, other);
    }
    /** @internal */
    '@@hash'() {
        return hashIterator(iterateFromFirst(this));
    }
    /** @internal */
    '@@unwrap'() {
        return _unwrap(this);
    }
    /** @internal */
    '@@unwrapInto'(target) {
        return unwrap(this, target);
    }
    /** @internal */
    '@@createUnwrapTarget'() {
        return {};
    }
    /** @internal */
    '@@get'(key) {
        return get(key, this);
    }
    /** @internal */
    '@@has'(key) {
        return has(key, this);
    }
    /** @internal */
    '@@set'(key, value) {
        return set(key, value, this);
    }
    /** @internal */
    '@@update'(updater, key) {
        return update(updater, key, this);
    }
    /** @internal */
    '@@verifyKey'(key) {
        return isDefined(key);
    }
    [Symbol.iterator]() {
        return iterateFromFirst(this);
    }
}
export function isRedBlackTree(arg) {
    return isObject(arg) && arg instanceof RedBlackTreeStructure;
}
export function createTree(compare, pctx) {
    return new RedBlackTreeStructure(selectContext(pctx), compare, NONE, 0);
}
export function cloneTree(tree, pctx) {
    if (isUndefined(pctx))
        pctx = isMutable(tree);
    return new RedBlackTreeStructure(selectContext(pctx), tree._compare, tree._root, tree._size);
}
//# sourceMappingURL=RedBlackTree.js.map